<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xmi="http://www.omg.org/XMI" xmlns:uuid="org.eclipse.emf.ecore.util.EcoreUtil" xmlns:xalan="http://xml.apache.org/xalan" exclude-result-prefixes="xalan xmi xsl uuid">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" xalan:indent-amount="3"/>
	<xsl:variable name="smallcase" select="'abcdefghijklmnopqrstuvwxyz'"/>
	<xsl:variable name="uppercase" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>
	<xsl:template match="DataService">
		<xsl:variable name="repositories" select="/DataService/Transaction/Command/MasterCatalog">
        </xsl:variable>
		<xmi:XMI xmi:version="2.0" xmlns:xmi="http://www.omg.org/XMI" xmlns:notation="http://www.eclipse.org/gmf/runtime/1.0.1/notation" xmlns:repositorymodel="http://www.tibco.com/mdm/repositorymodel/1.0">
		<xsl:variable name="uuid"><xsl:value-of select="uuid:generateUUID()"/></xsl:variable>
		<xsl:element name="repositorymodel:DataDomain">
                <xsl:attribute name="xmi:id"><xsl:value-of select="$uuid"/></xsl:attribute>                     
				<xsl:for-each select="$repositories">
					<xsl:element name="Repository">
					    <xsl:variable name="repositoryName" select="ExternalKeys/Key[@name='name']"></xsl:variable>
						<xsl:attribute name="xmi:type">repositorymodel:Repository</xsl:attribute>
						<xsl:attribute name="xmi:id"><xsl:value-of select="concat('_repository_',$repositoryName)"/></xsl:attribute>
						<xsl:attribute name="Name"><xsl:value-of select="$repositoryName"/></xsl:attribute>
						<xsl:attribute name="Description"><xsl:value-of select="EntityData/Attribute[@name='description']"/></xsl:attribute>
						<xsl:if test="count(EntityData/Attribute[@name='tableName']) &gt; 0">
							<xsl:attribute name="TableName"><xsl:value-of select="EntityData/Attribute[@name='tableName']"/></xsl:attribute>
						</xsl:if>
						<xsl:call-template name="AttributeGroup">
							<xsl:with-param name="repositoryName" select="$repositoryName"/>
						</xsl:call-template>	
					</xsl:element>
				</xsl:for-each>
				<!-- Relationships -->
				<xsl:for-each select="$repositories">
					<xsl:variable name="repositoryName" select="ExternalKeys/Key[@name='name']">
                    </xsl:variable>
					<xsl:for-each select="RelationshipData/Relationship">
						<xsl:if test="RelationType='RelationshipDefinition'">
							<xsl:if test="RelatedEntities/Entity">
								<xsl:for-each select="RelatedEntities/Entity">
									<xsl:element name="Relationship">
										<xsl:attribute name="xmi:type">repositorymodel:Relationship</xsl:attribute>
										<xsl:attribute name="xmi:id"><xsl:value-of select="concat('_relationship_', EntityData/Attribute[@name='name'])"/></xsl:attribute>
										<xsl:attribute name="Name"><xsl:value-of select="EntityData/Attribute[@name='name']"/></xsl:attribute>
										<xsl:attribute name="Description"><xsl:value-of select="EntityData/Attribute[@name='description']"/></xsl:attribute>
										<xsl:attribute name="ReverseName"><xsl:value-of select="EntityData/Attribute[@name='reverseName']"/></xsl:attribute>
										<xsl:attribute name="ReverseDescription"><xsl:value-of select="EntityData/Attribute[@name='reverseDescription']"/></xsl:attribute>
										<xsl:attribute name="sourceRepository"><xsl:value-of select="concat('_repository_',$repositoryName)"/></xsl:attribute>
										<xsl:attribute name="targetRepository"><xsl:value-of select="concat('_repository_',EntityData/Attribute[@name='targetCatalogName'])"/></xsl:attribute>
									</xsl:element>
								</xsl:for-each>
							</xsl:if>
						</xsl:if>
					</xsl:for-each>
				</xsl:for-each>
			</xsl:element> 
		</xmi:XMI>
	</xsl:template>
	
	<xsl:template name="AttributeGroup">
		<xsl:param name="repositoryName"/>
		<xsl:for-each select="RelationshipData/Relationship">
			<xsl:if test="RelationType='AttributeGroup'">
				<xsl:for-each select="RelatedEntities/Entity">
					<xsl:element name="AttributeGroup">
						<xsl:attribute name="xmi:type">repositorymodel:AttributeGroup</xsl:attribute>
						<xsl:attribute name="xmi:id"><xsl:value-of select="concat('_attributegroup_', $repositoryName, '_', ExternalKeys/Key[@name='name'])"/></xsl:attribute>
						<xsl:attribute name="Name"><xsl:value-of select="ExternalKeys/Key[@name='name']"/></xsl:attribute>
						<xsl:attribute name="Description"><xsl:value-of select="EntityData/Attribute[@name='Description']"/></xsl:attribute>
						<xsl:attribute name="Predefined"><xsl:value-of select="EntityData/Attribute[@name='Predefined']"/></xsl:attribute>
						<xsl:attribute name="ExpandByDefault"><xsl:value-of select="EntityData/Attribute[@name='ExpandByDefault']"/></xsl:attribute>
						<xsl:call-template name="Attributes">
							<xsl:with-param name="repositoryName" select="$repositoryName"/>
							<xsl:with-param name="AttributeGroupName" select="ExternalKeys/Key[@name='name']"/>
						</xsl:call-template>
					</xsl:element>
				</xsl:for-each>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	<xsl:template name="Attributes">
		<xsl:param name="repositoryName"/>
		<xsl:param name="AttributeGroupName"/>
		<xsl:for-each select="../../../../RelationshipData/Relationship">
			<xsl:if test="RelationType='CatalogAttribute'">
				<xsl:for-each select="RelatedEntities/Entity">
					<xsl:variable name="attrGroupName" select="RelationshipData/Relationship/RelatedEntities/AttributeGroup/ExternalKeys/Key"/>
					<xsl:if test="$attrGroupName = $AttributeGroupName">
						<xsl:element name="Attribute">
							<xsl:attribute name="xmi:type">repositorymodel:Attribute</xsl:attribute>
							<xsl:attribute name="xmi:id"><xsl:value-of select="concat('_attribute_', $repositoryName, '_', EntityData/Attribute[@name='Name'])"/></xsl:attribute>
							<xsl:attribute name="Name">
								<xsl:choose>
									<xsl:when test="EntityData/Attribute[@name='Name'] = 'PRODUCTID'">ID</xsl:when>
									<xsl:when test="EntityData/Attribute[@name='Name'] = 'PRODUCTIDEXT'">IDEXT</xsl:when>
									<xsl:otherwise><xsl:value-of select="EntityData/Attribute[@name='Name']"/></xsl:otherwise></xsl:choose>
							</xsl:attribute>
							<xsl:attribute name="Description"><xsl:value-of select="EntityData/Attribute[@name='Description']"/></xsl:attribute>
							<xsl:attribute name="DisplayName"><xsl:value-of select="EntityData/Attribute[@name='DisplayName']"/></xsl:attribute>
							<xsl:attribute name="DisplayInRecordList"><xsl:value-of select="EntityData/Attribute[@name='DisplayInRecordList']"/></xsl:attribute>
							<xsl:attribute name="QuickViewable"><xsl:value-of select="EntityData/Attribute[@name='QuickViewable']"/></xsl:attribute>
							<xsl:attribute name="Length"><xsl:choose><xsl:when test="translate(EntityData/Attribute[@name='Length'], ' ','')!=''"><xsl:value-of select="EntityData/Attribute[@name='Length']"/></xsl:when><xsl:otherwise>0</xsl:otherwise></xsl:choose></xsl:attribute>
							<xsl:attribute name="Position"><xsl:choose><xsl:when test="translate(EntityData/Attribute[@name='Position'], ' ','')!=''"><xsl:value-of select="EntityData/Attribute[@name='Position']"/></xsl:when><xsl:otherwise>0</xsl:otherwise></xsl:choose></xsl:attribute>
							<xsl:attribute name="ColumnName"><xsl:value-of select="EntityData/Attribute[@name='ColumnName']"/></xsl:attribute>
							<xsl:attribute name="Searchable"><xsl:value-of select="EntityData/Attribute[@name='Searchable']"/></xsl:attribute>
							<xsl:attribute name="Type">
								<xsl:variable name="datatype" select="EntityData/Attribute[@name='DataType']"/>
								<xsl:choose>
									<xsl:when test="$datatype= 'INTEGER'">Integer</xsl:when>
									<xsl:when test="$datatype= 'VARCHAR2'">String</xsl:when>
									<xsl:when test="$datatype= 'AMOUNT'">Amount</xsl:when>
									<xsl:when test="$datatype= 'BOOLEAN'">Boolean</xsl:when>
									<xsl:when test="$datatype= 'FILE'">File</xsl:when><xsl:when test="$datatype= 'DATE'">Date</xsl:when>
									<xsl:when test="$datatype= 'NUMBER'">Decimal</xsl:when>
									<xsl:when test="$datatype= 'CUSTOM_DECIMAL'">CustomDecimal</xsl:when></xsl:choose>
								</xsl:attribute>
							<xsl:attribute name="Help"><xsl:value-of select="EntityData/Attribute[@name='Help']"/></xsl:attribute>
							<xsl:attribute name="MultiValue">false</xsl:attribute>
						</xsl:element>
					</xsl:if>
				</xsl:for-each>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
</xsl:stylesheet>
