/*
 * Copyright (c) TIBCO Software Inc 2004, 2006. All rights reserved.
 */
package com.tibco.mdm.cim.repositorymodel.customimport.wizard;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.diagram.core.commands.CreateDiagramCommand;
import org.eclipse.gmf.runtime.emf.commands.core.command.AbstractTransactionalCommand;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.ui.dialogs.SelectionDialog;

import com.tibco.mdm.cim.repositorymodel.customimport.CustomImportPlugin;
import com.tibco.mdm.cim.repositorymodel.customimport.internal.Messages;
import com.tibco.mdm.repositorymodel.DataDomain;
import com.tibco.mdm.repositorymodel.diagram.edit.parts.DataDomainEditPart;
import com.tibco.mdm.repositorymodel.diagram.part.DataDomainDiagramEditorPlugin;
import com.tibco.xpd.importexport.ImportExportPlugin;
import com.tibco.xpd.resources.XpdResourcesPlugin;
import com.tibco.xpd.resources.projectconfig.ProjectConfig;
import com.tibco.xpd.resources.projectconfig.SpecialFolder;
import com.tibco.xpd.ui.importexport.importwizard.AbstractImportWizard;
import com.tibco.xpd.ui.importexport.importwizard.ImportSubTask;

/**
 * import wizard to import cim meta data files.
 * 
 * @author mbhardwa
 */
public class RepoImportWizard extends AbstractImportWizard {

    private static final String[] fileExtFilter = new String[] { "xml","XML" }; //$NON-NLS-1$	

    private URL[] xslts;

	private IContainer destinationContainer;

	private String DEST_FILE_EXT = "rep";	
	
	private URI uri;

	private String selectedFileName;

	private String selectedProjectPath;

    public RepoImportWizard() {
        super();    
        setWindowTitle(Messages.ImportWizard_ImportCIMWizardTitle);
        setWindowMessage(Messages.ImportWizard_ImportCIMWizardMessage);   
        
        //Generate diagram
        registerSubTask(new ImportSubTask() {
           
            /*
             * (non-Javadoc)
             * 
             * @see com.tibco.xpd.ui.importexport.importWizard.ImportSubTask#dispose()
             */
            public void dispose() {
                // Nothing to dispose
            }

            /*
             * (non-Javadoc)
             * 
             * @see com.tibco.xpd.ui.importexport.importWizard.ImportSubTask#perform(org.eclipse.core.runtime.SubProgressMonitor,
             *      java.io.File, org.eclipse.core.resources.IFile)
             */
            public void perform(SubProgressMonitor monitor, File inputFile,
                    IFile outputFile) throws CoreException {                 	            	
        		try {
					IContainer container = getDestinationContainer();
					setSelectedFileName(outputFile.getName().substring(0, outputFile.getName().lastIndexOf(".")));
					setSelectedProjectPath(container.getFullPath().toString());
					setUri(URI.createPlatformResourceURI(getFilePath(), true));

					// generate view
					postTransformationProcessing(monitor, outputFile);

				} catch (Exception e) {
					System.out.println(e.getStackTrace());
				}            					               
            }
        });
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.tibco.xpd.ui.importexport.importWizard.pages.ImportWizardPageIO.IImportWizardPageProvider#getFileExtensionFilter()
     */
    
    @Override
	protected void performOperation(IProgressMonitor monitor)
			throws CoreException, InterruptedException {    	   	
		Thread current = Thread.currentThread();
		ClassLoader oldLoader = current.getContextClassLoader();
		try {
			current.setContextClassLoader(getClass().getClassLoader());
			super.performOperation(monitor);
			
			} catch (Exception e) {				
				System.out.println(e.getStackTrace());
			}finally {			
			    current.setContextClassLoader(oldLoader);
		}		
	}
    
    public void setSelectedFileName(String selectedFileName) {
		this.selectedFileName = selectedFileName;
	}

	public void setSelectedProjectPath(String selectedProjectPath) {
		this.selectedProjectPath = selectedProjectPath;
	}

    public String[] getFileExtensionFilter() {
        return fileExtFilter;
    }

    
    private String getFilePath() {
    	return getSelectedProjectPath() + "/" + getSelectedFileName() + "." + DEST_FILE_EXT; 
	}
    
    public String getSelectedProjectPath() {
		return selectedProjectPath;
	}

	public String getSelectedFileName() {
		return selectedFileName;
	}

	/*
     * (non-Javadoc)
     * 
     * @see com.tibco.xpd.ui.importexport.utils.ImportExportTransformer.ITransformationStylesheetsProvider#getXslts()
     */
    public URL[] getXslts() {      	    	    
			try {
				if (xslts == null) {
				      xslts = new URL[] { CustomImportPlugin.getDefault().getCimToRepXsltUrl()};
				}	
			
			} catch (RuntimeException e) {
				e.printStackTrace();
			}
			return xslts;
  }

    
    public Map<String, String> getXsltParameters() {
        return null;
    }     
        
    public IStatus validateDestinationContainer(IContainer container) {
        IStatus status = null;
        if (container != null && container instanceof IFolder) {
            IFolder folder = (IFolder) container;
            ProjectConfig config = XpdResourcesPlugin.getDefault()
                    .getProjectConfig(folder.getProject());

            if (config != null && config.getSpecialFolders() != null) {
                SpecialFolder sFolder = config.getSpecialFolders()
                        .getFolderContainer(folder);

                if (sFolder != null
                        && sFolder
                                .getKind()
                                .equals("rep")) {
                    destinationContainer = container;
                    status = Status.OK_STATUS;
                }
            }
        }

        return status != null ? status : new Status(IStatus.ERROR,
                ImportExportPlugin.PLUGIN_ID, IStatus.OK,
                Messages.ImportWizard_destinationError_message, null);

    }
    
    @Override
    protected IContainer getDestinationContainer() {
    	return destinationContainer;
    }
    
    @Override
    protected String getImportFileExtension() {
        return DEST_FILE_EXT ;
    }

	public SelectionDialog getDestinationContainerSelectionDialog() {		

        return null;
	}
        
	private void postTransformationProcessing(IProgressMonitor monitor, IResource inputResource) {
		TransactionalEditingDomain ed = XpdResourcesPlugin
	        .getDefault().getEditingDomain();
	    AbstractTransactionalCommand command = new AbstractTransactionalCommand(ed,
	                "Generate Diagram",
	                Collections.EMPTY_LIST) {
	            protected CommandResult doExecuteWithResult(
	                    IProgressMonitor monitor, IAdaptable info)
	                    throws ExecutionException {
    		TransactionalEditingDomain ed = XpdResourcesPlugin.getDefault().getEditingDomain();	     			    	        		
    		Resource resource = ed.getResourceSet().getResource(getUri(), true);
    		//reload the resource as translated resource is not registred in the resource set.
    		//Fix for CIMS-534
	        resource.unload();
	        resource = ed.loadResource(getUri().toString());	       	        
	        if (resource != null && resource.getContents() != null) {
	        	try {
		        	// Only import xml files that have a Model
		        	if (hasModel(ed, resource)) {		        			                		                
		                EList<EObject> el = resource.getContents();
		                DataDomain model = null;
		                Diagram modelDiag = null;
		                for (EObject obj : el) {
		                	if (obj instanceof DataDomain) {
		                		model = (DataDomain)obj;
		                	} 
		                	else if (obj instanceof Diagram) {
		                		modelDiag = (Diagram)obj;
		                	}
		                }		              		               		                
		                
		                // Add the diagram for this model
		                CreateDiagramCommand cmd = new CreateDiagramCommand(
		                        ed,
		                        getLabel(),
		                        (EObject) model,
		                        DataDomainEditPart.MODEL_ID,DataDomainDiagramEditorPlugin.DIAGRAM_PREFERENCES_HINT);
		                IStatus status = cmd.execute(monitor, info);
		
		                if (status.isOK()) {
		                    Diagram value = (Diagram) cmd.getCommandResult().getReturnValue();
		                    value.setName(model.getName());		                   
		                    resource.getContents().add(value);		                    
		                } else {
		                    throw new ExecutionException(status
		                            .getMessage(), status.getException());
		                }
		                // Save the resource
		                try {
		                    resource.save(getSaveOptions());
		
		                    return CommandResult.newOKCommandResult();
		                } catch (IOException e) {
		                    throw new ExecutionException(Messages.Diagram_Generation_Error,e);
		                }
		
		        	}  
	        	} catch (Exception e) {	        			                
	                throw new ExecutionException(Messages.Diagram_Generation_Error,e);
	        	}
        	}
        	return CommandResult.newErrorCommandResult(Messages.Diagram_Generation_Error);
	    }

        };	
        try {
            OperationHistoryFactory.getOperationHistory().execute(command,
                    new SubProgressMonitor(monitor, 1), null);
        } catch (ExecutionException e) {
            
        }
	}
	
	 public URI getUri() {
		return uri;
	}

	public void setUri(URI uri) {
		this.uri = uri;
	}

	/**
     * @generated
     */
    public static Map getSaveOptions() {
        Map saveOptions = new HashMap();
        saveOptions.put(XMLResource.OPTION_ENCODING, "UTF-8"); //$NON-NLS-1$
        saveOptions.put(Resource.OPTION_SAVE_ONLY_IF_CHANGED,
                Resource.OPTION_SAVE_ONLY_IF_CHANGED_MEMORY_BUFFER);
        return saveOptions;
    }
	
	 /**
     * Check if the given resource contains an UML model. NOTE: This runs in a
     * read-only transaction.
     * 
     * @param res
     *            resource to check
     * @return <code>true</code> if the resource contains a model.
     * @throws InterruptedException
     */
    private boolean hasModel(TransactionalEditingDomain ed, final Resource res)
            throws InterruptedException {
        final Boolean[] ret = new Boolean[] { false };
        if (res != null && ed != null) {
            ed.runExclusive(new Runnable() {

                public void run() {
                    if (res.getContents() != null) {
                        for (EObject eo : res.getContents()) {
                            if (eo instanceof DataDomain) {
                                ret[0] = true;
                                break;
                            }
                        }
                    }
                }

            });
        }
        
        return ret[0];
    }
}
